@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: completion.c 261 2006-05-29 21:27:31Z titer $
+ * $Id: completion.c 1534 2007-03-05 23:03:38Z joshe $
  *
  * Copyright (c) 2005 Transmission authors and contributors
  *
@@ -30,9 +30,9 @@ tr_completion_t * tr_cpInit( tr_torrent_t * tor )
 
     cp                   = malloc( sizeof( tr_completion_t ) );
     cp->tor              = tor;
-    cp->blockBitfield    = malloc( ( tor->blockCount + 7 ) / 8 );
+    cp->blockBitfield    = tr_bitfieldNew( tor->blockCount );
     cp->blockDownloaders = malloc( tor->blockCount );
-    cp->pieceBitfield    = malloc( ( tor->info.pieceCount + 7 ) / 8 );
+    cp->pieceBitfield    = tr_bitfieldNew( tor->info.pieceCount );
     cp->missingBlocks    = malloc( tor->info.pieceCount * sizeof( int ) );
 
     tr_cpReset( cp );
@@ -42,11 +42,11 @@ tr_completion_t * tr_cpInit( tr_torrent_t * tor )
 
 void tr_cpClose( tr_completion_t * cp )
 {
-    free( cp->blockBitfield );
-    free( cp->blockDownloaders );
-    free( cp->pieceBitfield );
-    free( cp->missingBlocks );
-    free( cp );
+    tr_bitfieldFree( cp->blockBitfield );
+    free(            cp->blockDownloaders );
+    tr_bitfieldFree( cp->pieceBitfield );
+    free(            cp->missingBlocks );
+    free(            cp );
 }
 
 void tr_cpReset( tr_completion_t * cp )
@@ -55,9 +55,9 @@ void tr_cpReset( tr_completion_t * cp )
     int i;
 
     cp->blockCount = 0;
-    memset( cp->blockBitfield,    0, ( tor->blockCount + 7 ) / 8 );
+    tr_bitfieldClear( cp->blockBitfield );
     memset( cp->blockDownloaders, 0, tor->blockCount );
-    memset( cp->pieceBitfield,    0, ( tor->info.pieceCount + 7 ) / 8 );
+    tr_bitfieldClear( cp->pieceBitfield );
     for( i = 0; i < tor->info.pieceCount; i++ )
     {
         cp->missingBlocks[i] = tr_pieceCountBlocks( i );
@@ -85,12 +85,28 @@ uint64_t tr_cpLeftBytes( tr_completion_t * cp )
 }
 
 /* Pieces */
+int tr_cpPieceHasAllBlocks( tr_completion_t * cp, int piece )
+{
+    tr_torrent_t * tor = cp->tor;
+    int startBlock  = tr_pieceStartBlock( piece );
+    int endBlock    = startBlock + tr_pieceCountBlocks( piece );
+    int i;
+
+    for( i = startBlock; i < endBlock; i++ )
+    {
+        if( !tr_bitfieldHas( cp->blockBitfield, i ) )
+        {
+            return 0;
+        }
+    }    
+    return 1;
+}
 int tr_cpPieceIsComplete( tr_completion_t * cp, int piece )
 {
     return tr_bitfieldHas( cp->pieceBitfield, piece );
 }
 
-uint8_t * tr_cpPieceBitfield( tr_completion_t * cp )
+tr_bitfield_t * tr_cpPieceBitfield( tr_completion_t * cp )
 {
     return cp->pieceBitfield;
 }
@@ -110,6 +126,21 @@ void tr_cpPieceAdd( tr_completion_t * cp, int piece )
     tr_bitfieldAdd( cp->pieceBitfield, piece );
 }
 
+void tr_cpPieceRem( tr_completion_t * cp, int piece )
+{
+    tr_torrent_t * tor = cp->tor;
+    int startBlock, endBlock, i;
+
+    startBlock = tr_pieceStartBlock( piece );
+    endBlock   = startBlock + tr_pieceCountBlocks( piece );
+    for( i = startBlock; i < endBlock; i++ )
+    {
+        tr_cpBlockRem( cp, i );
+    }
+
+    tr_bitfieldRem( cp->pieceBitfield, piece );
+}
+
 /* Blocks */
 void tr_cpDownloaderAdd( tr_completion_t * cp, int block )
 {
@@ -164,12 +195,12 @@ void tr_cpBlockRem( tr_completion_t * cp, int block )
     tr_bitfieldRem( cp->blockBitfield, block );
 }
 
-uint8_t * tr_cpBlockBitfield( tr_completion_t * cp )
+tr_bitfield_t * tr_cpBlockBitfield( tr_completion_t * cp )
 {
     return cp->blockBitfield;
 }
 
-void tr_cpBlockBitfieldSet( tr_completion_t * cp, uint8_t * bitfield )
+void tr_cpBlockBitfieldSet( tr_completion_t * cp, tr_bitfield_t * bitfield )
 {
     tr_torrent_t * tor = cp->tor;
     int i, j;
@@ -200,6 +231,32 @@ void tr_cpBlockBitfieldSet( tr_completion_t * cp, uint8_t * bitfield )
     }
 }
 
+float tr_cpPercentBlocksInPiece( tr_completion_t * cp, int piece )
+{
+    tr_torrent_t * tor = cp->tor;
+    int i;
+    int blockCount, startBlock, endBlock;
+    int complete;
+    tr_bitfield_t * bitfield;
+    
+    blockCount  = tr_pieceCountBlocks( piece );
+    startBlock  = tr_pieceStartBlock( piece );
+    endBlock    = startBlock + blockCount;
+    complete    = 0;
+    
+    bitfield = cp->blockBitfield;
+
+    for( i = startBlock; i < endBlock; i++ )
+    {
+        if( tr_bitfieldHas( bitfield, i ) )
+        {
+            complete++;
+        }
+    }    
+
+    return (float)complete / (float)blockCount;
+}
+
 int tr_cpMissingBlockInPiece( tr_completion_t * cp, int piece )
 {
     tr_torrent_t * tor = cp->tor;
@@ -253,14 +310,17 @@ int tr_cpMostMissingBlockInPiece( tr_completion_t * cp, int piece,
         }
     }
 
-    if( poolSize < 1 )
+    if( poolSize > 0 )
     {
-        return -1;
+        ret = pool[0];
+        *downloaders = min;
+    }
+    else
+    {
+        ret = -1;
     }
 
-    ret = pool[0];
     free( pool );
-    *downloaders = min;
     return ret;
 }
 
