@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: utils.h 310 2006-06-09 19:53:35Z joshe $
+ * $Id: utils.h 1763 2007-04-20 02:05:07Z joshe $
  *
- * Copyright (c) 2005-2006 Transmission authors and contributors
+ * Copyright (c) 2005-2007 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -25,13 +25,12 @@
 #ifndef TR_UTILS_H
 #define TR_UTILS_H 1
 
-#define TR_MSG_ERR 1
-#define TR_MSG_INF 2
-#define TR_MSG_DBG 4
+void tr_msgInit( void );
+
 #define tr_err( a... ) tr_msg( TR_MSG_ERR, ## a )
 #define tr_inf( a... ) tr_msg( TR_MSG_INF, ## a )
 #define tr_dbg( a... ) tr_msg( TR_MSG_DBG, ## a )
-void tr_msg  ( int level, char * msg, ... );
+void tr_msg  ( int level, char * msg, ... ) PRINTF( 2, 3 );
 
 int  tr_rand ( int );
 
@@ -45,6 +44,39 @@ void * tr_memmem( const void *, size_t, const void *, size_t );
  **********************************************************************/
 int tr_mkdir( char * path );
 
+/***********************************************************************
+ * tr_strcasecmp
+ ***********************************************************************
+ * A case-insensitive strncmp()
+ **********************************************************************/
+#define tr_strcasecmp( ff, ss ) ( tr_strncasecmp( (ff), (ss), -1 ) )
+int tr_strncasecmp( const char * first, const char * second, int len );
+
+/***********************************************************************
+ * tr_sprintf
+ ***********************************************************************
+ * Appends to the end of a buffer using printf formatting,
+ * growing the buffer if needed
+ **********************************************************************/
+int tr_sprintf( char ** buf, int * used, int * max,
+                const char * format, ... ) PRINTF( 4, 5 );
+/* gee, it sure would be nice if BeOS had va_copy() */
+int tr_vsprintf( char **, int *, int *, const char *, va_list, va_list );
+/* this concatenates some binary data onto the end of a buffer */
+int tr_concat( char ** buf, int * used, int * max,
+               const char * data, int len );
+
+/***********************************************************************
+ * tr_dupstr
+ ***********************************************************************
+ * Creates a nul-terminated string 
+ **********************************************************************/
+char * tr_dupstr( const char * base, int len );
+
+int    tr_ioErrorFromErrno( void );
+
+char * tr_errorString( int code );
+
 /***********************************************************************
  * tr_date
  ***********************************************************************
@@ -71,25 +103,69 @@ static inline void tr_wait( uint64_t delay )
 #endif
 }
 
+struct tr_bitfield_s
+{
+    uint8_t * bits;
+    int       len;
+};
+
+/* note that the argument is how many bits are needed, not bytes */
+static inline tr_bitfield_t * tr_bitfieldNew( int bitcount )
+{
+    tr_bitfield_t * ret;
+
+    ret = calloc( 1, sizeof *ret );
+    if( NULL == ret )
+    {
+        return NULL;
+    }
+    ret->len = ( bitcount + 7 ) / 8;
+    ret->bits = calloc( ret->len, 1 );
+    if( NULL == ret->bits )
+    {
+        free( ret );
+        return NULL;
+    }
+
+    return ret;
+}
+
+static inline void tr_bitfieldFree( tr_bitfield_t * bitfield )
+{
+    if( bitfield )
+    {
+        free( bitfield->bits );
+        free( bitfield );
+    }
+}
+
+static inline void tr_bitfieldClear( tr_bitfield_t * bitfield )
+{
+    memset( bitfield->bits, 0, bitfield->len );
+}
+
 /***********************************************************************
  * tr_bitfieldHas
  **********************************************************************/
-static inline int tr_bitfieldHas( uint8_t * bitfield, int piece )
+static inline int tr_bitfieldHas( tr_bitfield_t * bitfield, int piece )
 {
-    return ( bitfield[ piece / 8 ] & ( 1 << ( 7 - ( piece % 8 ) ) ) );
+    assert( piece / 8 < bitfield->len );
+    return ( bitfield->bits[ piece / 8 ] & ( 1 << ( 7 - ( piece % 8 ) ) ) );
 }
 
 /***********************************************************************
  * tr_bitfieldAdd
  **********************************************************************/
-static inline void tr_bitfieldAdd( uint8_t * bitfield, int piece )
+static inline void tr_bitfieldAdd( tr_bitfield_t * bitfield, int piece )
 {
-    bitfield[ piece / 8 ] |= ( 1 << ( 7 - ( piece % 8 ) ) );
+    assert( piece / 8 < bitfield->len );
+    bitfield->bits[ piece / 8 ] |= ( 1 << ( 7 - ( piece % 8 ) ) );
 }
 
-static inline void tr_bitfieldRem( uint8_t * bitfield, int piece )
+static inline void tr_bitfieldRem( tr_bitfield_t * bitfield, int piece )
 {
-    bitfield[ piece / 8 ] &= ~( 1 << ( 7 - ( piece % 8 ) ) );
+    assert( piece / 8 < bitfield->len );
+    bitfield->bits[ piece / 8 ] &= ~( 1 << ( 7 - ( piece % 8 ) ) );
 }
 
 #define tr_blockPiece(a) _tr_blockPiece(tor,a)
